@@ -39,13 +39,14 @@ met:
 #define _GNU_SOURCE 1
 #define _BSD_SOURCE 1
 #define __EXTENSIONS__ 1
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
 #include <sys/types.h> /* inet_ntoa on freebsd */
+#include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <stdio.h>
+#include <string.h>
+#include <time.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <math.h>
@@ -115,6 +116,8 @@ static void mserv_cmd_set_genre(t_client *cl, const char *ru,
 				const char *line);
 static void mserv_cmd_set_year(t_client *cl, const char *ru,
 			       const char *line);
+static void mserv_cmd_set_volume(t_client *cl, const char *ru,
+                                 const char *line);
 static void mserv_cmd_set_albumauthor(t_client *cl, const char *ru,
 				      const char *line);
 static void mserv_cmd_set_albumname(t_client *cl, const char *ru,
@@ -336,6 +339,9 @@ t_cmds mserv_set_cmds[] = {
   { 1, level_priv, "YEAR", mserv_cmd_set_year,
     "Set the year of a track",
     "<album> <track> <0|year>" },
+  { 1, level_priv, "VOLUME", mserv_cmd_set_volume,
+    "Set the volume of a track (in percentage - 100 is normal)",
+    "<album> <track> <0-1000>" },
   { 1, level_priv, "ALBUMAUTHOR", mserv_cmd_set_albumauthor,
     "Set the author of an album",
     "<album> <author>" },
@@ -453,14 +459,14 @@ static void mserv_cmd_help(t_client *cl, const char *ru, const char *line)
 	if (cl->mode == mode_human) {
 	  snprintf(buffer, 1024, "[] %s\r\n[] Syntax: %s%s %s\r\n",
 		   cmdsptr->help, (cmdsptr == mserv_x_cmds ? "X " : 
-				   (cmdsptr == mserv_set_cmds ? "SET" : "")),
+				   (cmdsptr == mserv_set_cmds ? "SET " : "")),
 		   cmdsptr->name, cmdsptr->syntax);
 	  mserv_send(cl, buffer, 0);
 	} else {
 	  mserv_responsent(cl, "HELP", "%s", cmdsptr->name);
 	  snprintf(buffer, 1024, "%s\r\nSyntax: %s%s %s\r\n.\r\n",
 		   cmdsptr->help, (cmdsptr == mserv_x_cmds ? "X " : 
-				   (cmdsptr == mserv_set_cmds ? "SET" : "")),
+				   (cmdsptr == mserv_set_cmds ? "SET " : "")),
 		   cmdsptr->name, cmdsptr->syntax);
 	  mserv_send(cl, buffer, 0);
 	}
@@ -582,6 +588,8 @@ static void mserv_cmd_pass(t_client *cl, const char *ru, const char *line)
   }
   if (cl->mode != mode_computer)
     mserv_broadcast("CONNECT", "%s", cl->user);
+  mserv_log("%s: %s successfully logged in", inet_ntoa(cl->sin.sin_addr),
+            cl->user);
   cl->authed = 1;
   cl->userlevel = acl->userlevel;
   mserv_response(cl, "ACLSUCC", "%s", cl->user);
@@ -595,32 +603,37 @@ static void mserv_cmd_status(t_client *cl, const char *ru, const char *line)
   char token[16];
   char *a;
   int i;
-  time_t ago;
+  struct timeval now, ago;
 
   (void)ru;
   (void)line;
   if (mserv_playing.track) {
-    a = mserv_paused ? "STATPAU" : "STATPLA";
-    ago = time(NULL)-mserv_playingstart;
-    mserv_response(cl, a, "%s\t%d\t%d\t%d\t%d\t%s\t%s\t%d\t%d:%02d\t%s\t%.2f",
+    if (gettimeofday(&now, NULL) != 0) {
+      mserv_response(cl, "SERROR", "%s", "Failed to gettimeofday()");
+      return;
+    }
+    mserv_timersub(&now, &mserv_playing_start, &ago);
+    a = output_paused(mserv_output) ? "STATPAU" : "STATPLA";
+    mserv_response(cl, a, "%s\t%d\t%d\t%d\t%d\t%s\t%s\t%d\t%d:%02d.%d\t%s"
+                   "\t%.2f",
 		   mserv_getfilter(), mserv_filter_ok, mserv_filter_notok,
 		   mserv_playing.track->n_album,
 		   mserv_playing.track->n_track,
 		   mserv_playing.track->author,
-		   mserv_playing.track->name, mserv_playingstart,
-		   ago/60, ago % 60,
+		   mserv_playing.track->name, mserv_playing_start.tv_sec,
+		   ago.tv_sec/60, ago.tv_sec % 60, ago.tv_usec / 100000,
 		   mserv_random ? "ON" : "OFF", mserv_factor);
     if (cl->mode == mode_human) {
       for (i = 1; i <= 7; i++) {
 	sprintf(token, "STAT%d", i);
-	mserv_response(cl, token, "%s\t%d\t%d\t%d\t%d\t%s\t%s\t%d\t%d:%02d"
+	mserv_response(cl, token, "%s\t%d\t%d\t%d\t%d\t%s\t%s\t%d\t%d:%02d.%d"
 		       "\t%s\t%.2f",
 		       mserv_getfilter(), mserv_filter_ok, mserv_filter_notok,
 		       mserv_playing.track->n_album,
 		       mserv_playing.track->n_track,
 		       mserv_playing.track->author,
-		       mserv_playing.track->name, mserv_playingstart,
-		       ago/60, ago % 60,
+		       mserv_playing.track->name, mserv_playing_start.tv_sec,
+                       ago.tv_sec/60, ago.tv_sec % 60, ago.tv_usec / 100000,
 		       mserv_random ? "ON" : "OFF", mserv_factor);
       }
     }
@@ -957,13 +970,13 @@ static void mserv_cmd_tracks(t_client *cl, const char *ru, const char *line)
   }
   mserv_responsent(cl, "TRACKS", "%d\t%s\t%s", album->id, album->author,
 		   album->name);
-  for (i = 0; i < TRACKSPERALBUM; i++) {
+  for (i = 0; i < album->ntracks; i++) {
     if (album->tracks[i]) {
       rate = mserv_getrate(ru, album->tracks[i]);
       sprintf(bit, "%d/%d", album->tracks[i]->n_album,
 	      album->tracks[i]->n_track);
       if (cl->mode == mode_human) {
-	sprintf(buffer, "[] %6.6s %-1.1s %-20.20s %-40.40s %2ld:%02ld\r\n",
+	sprintf(buffer, "[] %7.7s %-1.1s %-20.20s %-39.39s %2ld:%02ld\r\n",
 		bit, rate && rate->rating ? mserv_ratestr(rate) : "-",
 		album->tracks[i]->author, album->tracks[i]->name,
 		(album->tracks[i]->duration / 100) / 60,
@@ -1119,7 +1132,7 @@ static void mserv_cmd_queue(t_client *cl, const char *ru, const char *line)
   unsigned int i, j, k, numtracks;
   t_rating *rate;
   int random, mode;
-  unsigned char tracktally[TRACKSPERALBUM];
+  unsigned char *tracktally;
 
   if (!*line) {
     if (!mserv_queue) {
@@ -1132,8 +1145,8 @@ static void mserv_cmd_queue(t_client *cl, const char *ru, const char *line)
       if (cl->mode == mode_human) {
 	sprintf(bit, "%d/%d", q->supinfo.track->n_album,
 		q->supinfo.track->n_track);
-	sprintf(buffer, "[] %-10.10s %6.6s %-1.1s %-20.20s "
-		"%-30.30s%2ld:%02ld\r\n",
+	sprintf(buffer, "[] %-10.10s %7.7s %-1.1s %-20.20s "
+		"%-29.29s%2ld:%02ld\r\n",
 		q->supinfo.user, bit,
 		rate && rate->rating ? mserv_ratestr(rate) : "-",
 		q->supinfo.track->author, q->supinfo.track->name,
@@ -1193,34 +1206,38 @@ static void mserv_cmd_queue(t_client *cl, const char *ru, const char *line)
       mserv_responsent(cl, "QADDR", NULL);
     if (random) {
       numtracks = 0;
-      for (i = 0; i < TRACKSPERALBUM; i++) {
-	if (album->tracks[i]) {
-	  numtracks++;
-	  tracktally[i] = 0;
-	} else {
-	  tracktally[i] = 1;
-	}
+      if ((tracktally = malloc(album->ntracks)) == NULL) {
+        mserv_log("Out of memory creating tally for album randomisation");
+      } else {
+        /* this code seems to think tracks could be missing in an album?! */
+        for (i = 0; i < album->ntracks; i++) {
+          if (album->tracks[i]) {
+            numtracks++;
+            tracktally[i] = 0;
+          } else {
+            tracktally[i] = 1;
+          }
+        }
+        for (i = 0; i < numtracks; i++) {
+          j = ((double)(numtracks-i))*rand()/(RAND_MAX+1.0);
+          for (k = 0; k < album->ntracks; k++) {
+            if (!tracktally[k]) {
+              if (!j--)
+                break;
+            }
+          }
+          if (k >= album->ntracks || tracktally[k]) {
+            /* assertion - should never happen */
+            mserv_log("Internal error in randomising album (%d)", k);
+            break;
+          }
+          tracktally[k] = 1;
+          mserv_cmd_queue_sub(cl, album, k+1, 0);
+        }
+        free(tracktally);
       }
-      for (i = 0; i < numtracks; i++) {
-	mserv_log("LOOP: %d of %d", i, numtracks);
-	j = ((double)(numtracks-i))*rand()/(RAND_MAX+1.0);
-	mserv_log("  -->%d", j);
-	for (k = 0; k < TRACKSPERALBUM; k++) {
-	  if (!tracktally[k]) {
-	    if (!j--)
-	      break;
-	  }
-	}
-	if (k >= TRACKSPERALBUM || tracktally[k]) {
-	  /* should never happen */
-	  mserv_log("Internal error in randomising album (%d)", k);
-	  exit(1);
-	}
-	tracktally[k] = 1;
-	mserv_cmd_queue_sub(cl, album, k+1, 0);
-      }	
     } else {     
-      for (i = 0; i < TRACKSPERALBUM; i++) {
+      for (i = 0; i < album->ntracks; i++) {
 	if (album->tracks[i])
 	  mserv_cmd_queue_sub(cl, album, i+1, 0);
       }
@@ -1235,7 +1252,7 @@ static void mserv_cmd_queue(t_client *cl, const char *ru, const char *line)
     mserv_response(cl, "NAN", NULL);
     return;
   }
-  if (n_track < 1 || n_track >= TRACKSPERALBUM || !album->tracks[n_track-1]) {
+  if (n_track < 1 || n_track >= album->ntracks || !album->tracks[n_track-1]) {
     mserv_response(cl, "NOTRACK", NULL);
     return;
   }
@@ -1281,8 +1298,8 @@ static int mserv_cmd_queue_sub(t_client *cl, t_album *album, int n_track,
   for (client = mserv_clients; client; client = client->next) {
     rate = mserv_getrate(client->user, album->tracks[i]);
     if (client->mode == mode_human) {
-      sprintf(buffer, "[] %-10.10s %6.6s %-1.1s %-20.20s "
-	      "%-30.30s%2ld:%02ld\r\n", cl->user, bit,
+      sprintf(buffer, "[] %-10.10s %7.7s %-1.1s %-20.20s "
+	      "%-29.29s%2ld:%02ld\r\n", cl->user, bit,
 	      rate && rate->rating ? mserv_ratestr(rate) : "-",
 	      album->tracks[i]->author, album->tracks[i]->name,
 	      (album->tracks[i]->duration / 100) / 60,
@@ -1305,7 +1322,7 @@ static void mserv_cmd_play(t_client *cl, const char *ru, const char *line)
 {
   (void)ru;
   (void)line;
-  if (mserv_playing.track && mserv_paused) {
+  if (output_paused(mserv_output)) {
     mserv_resumeplay();
     mserv_broadcast("RESUME", "%s\t%d\t%d\t%s\t%s", cl->user,
 		    mserv_playing.track->n_album,
@@ -1329,9 +1346,10 @@ static void mserv_cmd_play(t_client *cl, const char *ru, const char *line)
 		   mserv_playing.track->name);
     return;
   }
-  if (mserv_playnext()) {
+  if (mserv_player_playnext()) {
     mserv_response(cl, "NOMORE", NULL);
   } else {
+    output_start(mserv_output);
     if (cl->mode != mode_human) {
       /* humans will already have seen broadcast */
       mserv_response(cl, "STARTED", "%d\t%d\t%s\t%s",
@@ -1346,7 +1364,7 @@ static void mserv_cmd_stop(t_client *cl, const char *ru, const char *line)
 {
   (void)ru;
   (void)line;
-  if (mserv_playing.track) {
+  if (!output_stopped(mserv_output)) {
     mserv_broadcast("STOPPED", "%s\t%d\t%d\t%s\t%s", cl->user,
 		    mserv_playing.track->n_album, mserv_playing.track->n_track,
 		    mserv_playing.track->author, mserv_playing.track->name);
@@ -1367,11 +1385,11 @@ static void mserv_cmd_pause(t_client *cl, const char *ru, const char *line)
 {
   (void)ru;
   (void)line;
-  if (mserv_paused) {
+  if (output_paused(mserv_output)) {
     mserv_response(cl, "APAUSED", NULL);
     return;
   }
-  if (mserv_playing.track || mserv_playnextat) {
+  if (mserv_playing.track) {
     mserv_pauseplay(cl);
     if (cl->mode != mode_human) {
       /* humans will already have seen broadcast */
@@ -1386,20 +1404,23 @@ static void mserv_cmd_next(t_client *cl, const char *ru, const char *line)
 {
   (void)ru;
   (void)line;
+
   if (mserv_playing.track)
     mserv_broadcast("SKIP", "%s", cl->user);
-  if (mserv_playnext()) {
+  mserv_abortplay();
+  if (mserv_player_playnext()) {
     mserv_broadcast("FINISH", NULL);
     if (cl->mode != mode_human)
       mserv_response(cl, "NOMORE", NULL);
-  } else {
-    if (cl->mode != mode_human) {
-      /* humans will already have seen broadcast */
-      mserv_response(cl, "NEXT", "%d\t%d\t%s\t%s",
-		     mserv_playing.track->n_album,
-		     mserv_playing.track->n_track, mserv_playing.track->author,
-		     mserv_playing.track->name);
-    }
+    return;
+  }
+  output_start(mserv_output);
+  if (cl->mode != mode_human) {
+    /* humans will already have seen broadcast */
+    mserv_response(cl, "NEXT", "%d\t%d\t%s\t%s",
+                   mserv_playing.track->n_album,
+                   mserv_playing.track->n_track, mserv_playing.track->author,
+                   mserv_playing.track->name);
   }
 }
 
@@ -1603,8 +1624,8 @@ static void mserv_cmd_top(t_client *cl, const char *ru, const char *line)
     rate = mserv_getrate(ru, track);
     if (cl->mode == mode_human) {
       sprintf(bit, "%d/%d", track->n_album, track->n_track);
-      sprintf(buffer, "[] %5.2f%%     %6.6s %-1.1s %-20.20s "
-	      "%-30.30s%2ld:%02ld\r\n",
+      sprintf(buffer, "[] %5.2f%%     %7.7s %-1.1s %-20.20s "
+	      "%-29.29s%2ld:%02ld\r\n",
 	      prob > 0.9999 ? 99.99 : 100*prob, bit,
 	      rate && rate->rating ? mserv_ratestr(rate) : "-",
 	      track->author, track->name,
@@ -1625,6 +1646,42 @@ static void mserv_cmd_top(t_client *cl, const char *ru, const char *line)
     mserv_send(cl, ".\r\n", 0);
 }
 
+#if ENGINE != local
+
+static void mserv_cmd_volume(t_client *cl, const char *ru, const char *line)
+{
+  int val;
+
+  (void)ru;
+  if (*line && cl->userlevel == level_guest) {
+    mserv_response(cl, "ACLFAIL", NULL);
+  } else if ((val = mserv_outputvolume(cl, line)) != -1) {
+    if (!*line) {
+      mserv_response(cl, "VOLCUR", "%d", val);
+    } else {
+      mserv_broadcast("VOLSET", "%d\t%s", val, cl->user);
+      if (cl->mode != mode_human)
+	mserv_response(cl, "VOLREP", "%d", val);
+    }
+  }
+}
+
+static void mserv_cmd_bass(t_client *cl, const char *ru, const char *line)
+{
+  (void)ru;
+  (void)line;
+  mserv_response(cl, "NOTIMPL", NULL);
+}
+
+static void mserv_cmd_treble(t_client *cl, const char *ru, const char *line)
+{
+  (void)ru;
+  (void)line;
+  mserv_response(cl, "NOTIMPL", NULL);
+}
+
+#else
+
 static void mserv_cmd_volume(t_client *cl, const char *ru, const char *line)
 {
 #ifdef SOUNDCARD
@@ -1639,7 +1696,7 @@ static void mserv_cmd_volume(t_client *cl, const char *ru, const char *line)
     } else {
       mserv_broadcast("VOLSET", "%d\t%s", val & 0xFF, cl->user);
       if (cl->mode != mode_human)
-	mserv_response(cl, "VOLREP", "%d", val & 0xFF);
+        mserv_response(cl, "VOLREP", "%d", val & 0xFF);
     }
   }
 #else
@@ -1697,6 +1754,8 @@ static void mserv_cmd_treble(t_client *cl, const char *ru, const char *line)
 #endif
 }
 
+#endif
+
 static void mserv_cmd_say(t_client *cl, const char *ru, const char *line)
 {
   (void)ru;
@@ -1872,7 +1931,7 @@ static void mserv_cmd_set_genre(t_client *cl, const char *ru,
       mserv_response(cl, "NOALBUM", NULL);
       return;
     }
-    for (i = 0; i < TRACKSPERALBUM; i++) {
+    for (i = 0; i < album->ntracks; i++) {
       if (album->tracks[i]) {
 	/* altertrack invalidates track pointer */
 	if (mserv_altertrack(album->tracks[i], NULL, NULL, str[n-1],
@@ -1922,6 +1981,10 @@ static void mserv_cmd_set_year(t_client *cl, const char *ru, const char *line)
     mserv_response(cl, "NAN", NULL);
     return;
   }
+  if (year < 100 || year > 1000) {
+    mserv_response(cl, "NAN", NULL);
+    return;
+  }
   if ((track = mserv_gettrack(n_album, n_track)) == NULL) {
     mserv_response(cl, "NOTRACK", NULL);
     return;
@@ -1938,6 +2001,57 @@ static void mserv_cmd_set_year(t_client *cl, const char *ru, const char *line)
   mserv_savechanges();
 }
 
+static void mserv_cmd_set_volume(t_client *cl, const char *ru, const char *line)
+{
+  char linespl[LINEBUFLEN];
+  char *str[4];
+  unsigned int n_album, n_track, volume;
+  char *end;
+  t_track *track;
+
+  /* <volume> */
+
+  (void)ru;
+  strcpy(linespl, line);
+  if (mserv_split(str, 3, linespl, " ") != 3) {
+    mserv_response(cl, "BADPARM", NULL);
+    return;
+  }
+  n_album = strtol(str[0], &end, 10);
+  if (!*str[0] || *end) {
+    mserv_response(cl, "NAN", NULL);
+    return;
+  }
+  n_track = strtol(str[1], &end, 10);
+  if (!*str[1] || *end) {
+    mserv_response(cl, "NAN", NULL);
+    return;
+  }
+  volume = strtol(str[2], &end, 10);
+  if (!*str[2] || *end) {
+    mserv_response(cl, "NAN", NULL);
+    return;
+  }
+  if (volume < 0 || volume > 1000) {
+    mserv_response(cl, "NAN", NULL);
+    return;
+  }
+  if ((track = mserv_gettrack(n_album, n_track)) == NULL) {
+    mserv_response(cl, "NOTRACK", NULL);
+    return;
+  }
+  track = mserv_checkdisk_track(track);
+  track->volume = volume;
+  track->modified = 1;
+  mserv_broadcast("VOLUME", "%s\t%d\t%d\t%s\t%s\t%d", cl->user, track->n_album,
+		  track->n_track, track->author, track->name, volume);
+  if (cl->mode != mode_human)
+    mserv_response(cl, "VOLUMER", "%s\t%d\t%d\t%s\t%s\t%d", cl->user,
+		   track->n_album, track->n_track, track->author,
+		   track->name, volume);
+  mserv_savechanges();
+}
+
 static void mserv_cmd_set_albumauthor(t_client *cl, const char *ru, const char *line)
 {
   char linespl[LINEBUFLEN];
@@ -2039,7 +2153,7 @@ static void mserv_cmd_history(t_client *cl, const char *ru, const char *line)
     if (cl->mode == mode_human) {
       sprintf(bit, "%d/%d", mserv_history[i]->track->n_album,
 	      mserv_history[i]->track->n_track);
-      sprintf(buffer, "[] %-10.10s %6.6s %-1.1s %-20.20s %-35.35s\r\n",
+      sprintf(buffer, "[] %-10.10s %7.7s %-1.1s %-20.20s %-34.34s\r\n",
 	      mserv_history[i]->user, bit,
 	      rate && rate->rating ? mserv_ratestr(rate) : "-",
 	      mserv_history[i]->track->author, mserv_history[i]->track->name);
@@ -2143,7 +2257,7 @@ static void mserv_cmd_rate(t_client *cl, const char *ru, const char *line)
     if (cl->mode == mode_human) {
       ratetoo = 0;
       for (album = mserv_albums; album; album = album->next) {
-	for (i = 0; i < TRACKSPERALBUM; i++) {
+	for (i = 0; i < album->ntracks; i++) {
 	  track2 = album->tracks[i];
 	  if (track2 && track2 != track &&
 	      !stricmp(track2->author, track->author) &&
@@ -2162,8 +2276,8 @@ static void mserv_cmd_rate(t_client *cl, const char *ru, const char *line)
 	      ratetoo = 1;
 	    }
 	    sprintf(bit, "%d/%d", track2->n_album, track2->n_track);
-	    sprintf(buffer, "[]            %6.6s %-1.1s %-20.20s "
-		    "%-30.30s%ld:%02ld\r\n",
+	    sprintf(buffer, "[]            %7.7s %-1.1s %-20.20s "
+		    "%-29.29s%ld:%02ld\r\n",
 		    bit, rate2 && rate2->rating ? mserv_ratestr(rate2) : "-",
 		    track2->author, track2->name,
 		    (track2->duration / 100) / 60,
@@ -2180,7 +2294,7 @@ static void mserv_cmd_rate(t_client *cl, const char *ru, const char *line)
     }
     rate = NULL;
     total = 0;
-    for (i = 0; i < TRACKSPERALBUM; i++) {
+    for (i = 0; i < album->ntracks; i++) {
       if (album->tracks[i]) {
 	if ((rate2 = mserv_getrate(cl->user, album->tracks[i])) == NULL ||
 	    rate2->rating == 0) { /* 0 means heard, not rated */
@@ -2225,7 +2339,7 @@ static void mserv_cmd_check(t_client *cl, const char *ru, const char *line)
     return;
   }
   for (author = mserv_authors; author; author = author->next) {
-    for (i = 0; i < TRACKSPERALBUM-1; i++) {
+    for (i = 0; i < author->ntracks; i++) {
       if ((track1 = author->tracks[i]) && (track2 = author->tracks[i+1])) {
 	if (!stricmp(track1->author, track2->author) &&
 	    !stricmp(track1->name, track2->name)) {
@@ -2242,16 +2356,16 @@ static void mserv_cmd_check(t_client *cl, const char *ru, const char *line)
 	  }
 	  if (cl->mode == mode_human) {
 	    sprintf(bit, "%d/%d", track1->n_album, track1->n_track);
-	    sprintf(buffer, "[]            %6.6s %-1.1s %-20.20s "
-		    "%-30.30s%2ld:%02ld\r\n",
+	    sprintf(buffer, "[]            %7.7s %-1.1s %-20.20s "
+		    "%-29.29s%2ld:%02ld\r\n",
 		    bit, rate1 && rate1->rating ? mserv_ratestr(rate1) : "-",
 		    track1->author, track1->name,
 		    (track1->duration / 100) / 60,
 		    (track1->duration / 100) % 60);
 	    mserv_send(cl, buffer, 0);
 	    sprintf(bit, "%d/%d", track2->n_album, track2->n_track);
-	    sprintf(buffer, "[]            %6.6s %-1.1s %-20.20s "
-		    "%-30.30s%2ld:%02ld\r\n",
+	    sprintf(buffer, "[]            %7.7s %-1.1s %-20.20s "
+		    "%-29.29s%2ld:%02ld\r\n",
 		    bit, rate2 && rate2->rating ? mserv_ratestr(rate2) : "-",
 		    track2->author, track2->name,
 		    (track2->duration / 100) / 60,
@@ -2300,7 +2414,7 @@ static void mserv_cmd_search(t_client *cl, const char *ru, const char *line)
     return;
   }
   for (author = mserv_authors; author; author = author->next) {
-    for (i = 0; i < TRACKSPERALBUM; i++) {
+    for (i = 0; i < author->ntracks; i++) {
       if ((track = author->tracks[i])) {
 	if ((stristr(track->author, line)) ||
 	    (stristr(track->name, line))) {
@@ -2311,8 +2425,8 @@ static void mserv_cmd_search(t_client *cl, const char *ru, const char *line)
 	  rate = mserv_getrate(ru, track);
 	  if (cl->mode == mode_human) {
 	    sprintf(bit, "%d/%d", track->n_album, track->n_track);
-	    sprintf(buffer, "[]            %6.6s %-1.1s %-20.20s "
-		    "%-30.30s%2ld:%02ld\r\n", bit,
+	    sprintf(buffer, "[]            %7.7s %-1.1s %-20.20s "
+		    "%-29.29s%2ld:%02ld\r\n", bit,
 		    rate && rate->rating ? mserv_ratestr(rate) : "-",
 		    track->author, track->name,
 		    (track->duration / 100) / 60,
@@ -2394,7 +2508,7 @@ static void mserv_cmd_searchf(t_client *cl, const char *ru, const char *line)
     return;
   }
   for (author = mserv_authors; author; author = author->next) {
-    for (i = 0; i < TRACKSPERALBUM; i++) {
+    for (i = 0; i < author->ntracks; i++) {
       if ((track = author->tracks[i])) {
 	if (filter_check(line, track) == 1) {
 	  if (!f) {
@@ -2404,8 +2518,8 @@ static void mserv_cmd_searchf(t_client *cl, const char *ru, const char *line)
 	  rate = mserv_getrate(ru, track);
 	  if (cl->mode == mode_human) {
 	    sprintf(bit, "%d/%d", track->n_album, track->n_track);
-	    sprintf(buffer, "[]            %6.6s %-1.1s %-20.20s "
-		    "%-30.30s%2ld:%02ld\r\n", bit,
+	    sprintf(buffer, "[]            %7.7s %-1.1s %-20.20s "
+		    "%-29.29s%2ld:%02ld\r\n", bit,
 		    rate && rate->rating ? mserv_ratestr(rate) : "-",
 		    track->author, track->name,
 		    (track->duration / 100) / 60,
@@ -2486,7 +2600,7 @@ static void mserv_cmd_info(t_client *cl, const char *ru, const char *line)
         return;
       }
       ttime = 0;
-      for (i = 0; i < TRACKSPERALBUM; i++) {
+      for (i = 0; i < album->ntracks; i++) {
 	if (album->tracks[i])
 	  ttime+= album->tracks[i]->duration;
       }
@@ -2535,20 +2649,20 @@ static void mserv_cmd_info(t_client *cl, const char *ru, const char *line)
   snprintf(year, sizeof(year), "%d", track->year);
   mserv_response(cl, "INFT",
 		 "%d\t%d\t%s\t%s\t%s\t%s\t%s\t%d\t%s\t%.1f\t%.1f\t"
-		 "%s\t%s\t%s\t%d:%02d.%d\t%s",
+		 "%s\t%s\t%s\t%d:%02d.%d\t%s\t%d",
 		 track->n_album, track->n_track, album->author, album->name,
 		 track->author, track->name, track->year ? year : "unknown",
 		 track->lastplay, ago, 100*track->prating, 100*track->rating,
 		 mserv_ratestr(rate), track->genres,
 		 track->filterok ? "included" : "excluded",
 		 (track->duration/100)/60, (track->duration/100) % 60,
-		 (track->duration/10) % 10, track->miscinfo);
+		 (track->duration/10) % 10, track->miscinfo, track->volume);
   if (cl->mode == mode_human) {
-    for (i = 1; i <= 11; i++) {
+    for (i = 1; i <= 12; i++) {
       sprintf(token, "INFT%d", i);
       mserv_response(cl, token,
 		     "%d\t%d\t%s\t%s\t%s\t%s\t%s\t%d\t%s\t%.1f\t%.1f\t"
-		     "%s\t%s\t%s\t%d:%02d.%d\t%s",
+		     "%s\t%s\t%s\t%d:%02d.%d\t%s\t%d",
 		     track->n_album, track->n_track, album->author,
 		     album->name, track->author, track->name,
 		     track->year ? year : "unknown", track->lastplay,
@@ -2556,7 +2670,8 @@ static void mserv_cmd_info(t_client *cl, const char *ru, const char *line)
 		     mserv_ratestr(rate), track->genres,
 		     track->filterok ? "included" : "excluded",
 		     (track->duration/100)/60, (track->duration/100) % 60,
-		     (track->duration/10) % 10, track->miscinfo);
+                     (track->duration/10) % 10, track->miscinfo,
+                     track->volume);
     }
   }
 }
@@ -2574,7 +2689,7 @@ static void mserv_cmd_x_authors(t_client *cl, const char *ru,
   for (author = mserv_authors; author; author = author->next) {
     total = 0;
     rated = 0;
-    for (i = 0; i < TRACKSPERALBUM; i++) {
+    for (i = 0; i < author->ntracks; i++) {
       if (author->tracks[i]) {
 	total++;
 	rate = mserv_getrate(ru, author->tracks[i]);
@@ -2633,7 +2748,7 @@ static void mserv_cmd_x_authorinfo(t_client *cl, const char *ru,
     if (author->id == n_author) {
       total = 0;
       rated = 0;
-      for (i = 0; i < TRACKSPERALBUM; i++) {
+      for (i = 0; i < author->ntracks; i++) {
 	if (author->tracks[i]) {
 	  total++;
 	  rate = mserv_getrate(ru, author->tracks[i]);
@@ -2672,13 +2787,13 @@ static void mserv_cmd_x_authortracks(t_client *cl, const char *ru,
   for (author = mserv_authors; author; author = author->next) {
     if (author->id == n_author) {
       mserv_responsent(cl, "AUTHTRK", "%d\t%s", author->id, author->name);
-      for (i = 0; i < TRACKSPERALBUM; i++) {
+      for (i = 0; i < author->ntracks; i++) {
 	if (author->tracks[i]) {
 	  rate = mserv_getrate(ru, author->tracks[i]);
 	  sprintf(bit, "%d/%d", author->tracks[i]->n_album,
 		  author->tracks[i]->n_track);
 	  if (cl->mode == mode_human) {
-	    sprintf(buffer, "[] %6.6s %-1.1s %-20.20s %-45.45s\r\n", bit,
+	    sprintf(buffer, "[] %7.7s %-1.1s %-20.20s %-44.44s\r\n", bit,
 		    rate && rate->rating ? mserv_ratestr(rate) : "-",
 		    author->tracks[i]->author, author->tracks[i]->name);
 	    mserv_send(cl, buffer, 0);
@@ -2835,20 +2950,21 @@ static void mserv_cmd_shutdown(t_client *cl, const char *ru, const char *line)
 
 static void mserv_cmd_gap(t_client *cl, const char *ru, const char *line)
 {
-  int delay;
+  double delay;
   char *end;
 
   (void)ru;
   if (!*line) {
-    mserv_response(cl, "GAPCUR", "%d", mserv_setgap(-1));
+    mserv_response(cl, "GAPCUR", "%.1f", mserv_getgap());
     return;
   }    
-  delay = strtol(line, &end, 10);
+  delay = strtod(line, &end);
   if (!*line || *end || delay < 0 || delay > 3600) {
     mserv_response(cl, "NAN", NULL);
     return;
   }
-  mserv_broadcast("DELAY", "%s\t%d", cl->user, mserv_setgap(delay));
+  mserv_setgap(delay);
+  mserv_broadcast("DELAY", "%s\t%.1f", cl->user, delay);
   if (cl->mode != mode_human)
     mserv_response(cl, "DELAYR", NULL);
 }
@@ -2865,17 +2981,17 @@ static void mserv_cmd_x_genres(t_client *cl, const char *ru, const char *line)
   mserv_responsent(cl, "GENRES", NULL);
   for (genre = mserv_genres; genre; genre = genre->next) {
     rated = 0;
-    for (ui = 0; ui < genre->total; ui++) {
+    for (ui = 0; ui < genre->ntracks; ui++) {
       rate = mserv_getrate(ru, genre->tracks[ui]);
       if (rate && rate->rating) /* rate->rating being 0 means 'heard' */
 	rated++;
     }
     if (cl->mode == mode_human) {
       sprintf(buffer, "[] %3d %-62.62s %4d %4d\r\n", genre->id,
-	      genre->name, rated, genre->total);
+	      genre->name, rated, genre->ntracks);
     } else {
       sprintf(buffer, "%d\t%s\t%d\t%d\r\n", genre->id, genre->name,
-	      genre->total, rated);
+	      genre->ntracks, rated);
     }
     mserv_send(cl, buffer, 0);
   }
@@ -2920,13 +3036,13 @@ static void mserv_cmd_x_genreinfo(t_client *cl, const char *ru,
   for (genre = mserv_genres; genre; genre = genre->next) {
     if (genre->id == n_genre) {
       rated = 0;
-      for (ui = 0; ui < genre->total; ui++) {
+      for (ui = 0; ui < genre->ntracks; ui++) {
 	rate = mserv_getrate(ru, genre->tracks[ui]);
 	if (rate && rate->rating) /* rate->rating being 0 means 'heard' */
 	  rated++;
       }
       mserv_response(cl, "GENINF", "%d\t%s\t%d\t%d", genre->id, genre->name,
-		     genre->total, rated);
+		     genre->ntracks, rated);
       return;
     }
   }
@@ -2956,13 +3072,13 @@ static void mserv_cmd_x_genretracks(t_client *cl, const char *ru,
   for (genre = mserv_genres; genre; genre = genre->next) {
     if (genre->id == n_genre) {
       mserv_responsent(cl, "GENTRK", "%d\t%s\t%d", genre->id, genre->name,
-		       genre->total);
-      for (ui = 0; ui < genre->total; ui++) {
+		       genre->ntracks);
+      for (ui = 0; ui < genre->ntracks; ui++) {
 	rate = mserv_getrate(ru, genre->tracks[ui]);
 	sprintf(bit, "%d/%d", genre->tracks[ui]->n_album,
 		genre->tracks[ui]->n_track);
 	if (cl->mode == mode_human) {
-	  sprintf(buffer, "[] %6.6s %-1.1s %-20.20s %-45.45s\r\n", bit,
+	  sprintf(buffer, "[] %7.7s %-1.1s %-20.20s %-44.44s\r\n", bit,
 		  rate && rate->rating ? mserv_ratestr(rate) : "-",
 		  genre->tracks[ui]->author, genre->tracks[ui]->name);
 	  mserv_send(cl, buffer, 0);
