@@ -49,6 +49,9 @@ met:
 #include "conf.h"
 #include "opt.h"
 
+const char *opt_default_icecast_output = NULL;
+const char *opt_default_icecast_bitrate = NULL;
+const char *opt_path_distconf = NULL;
 const char *opt_path_acl = NULL;
 const char *opt_path_webacl = NULL;
 const char *opt_path_logfile = NULL;
@@ -59,11 +62,12 @@ const char *opt_path_idea = NULL;
 const char *opt_path_mixer = NULL;
 const char *opt_path_language = NULL;
 unsigned int opt_port = 4444;
-unsigned int opt_gap = 1;
+double opt_gap = 1;
 unsigned int opt_play = 0;
 unsigned int opt_random = 0;
 double opt_factor = 0.6;
-const char *opt_player = NULL;
+double opt_rate_unheard = 0.55;
+double opt_rate_unrated = 0.50;
 const char *opt_filter = NULL;
 
 typedef struct {
@@ -74,6 +78,11 @@ typedef struct {
 } t_opts;
 
 t_opts opt_opts[] = {
+  { "default_icecast_output", "string", &opt_default_icecast_output,
+    "http://source:password@localhost:8000/mserv.ogg" },
+  { "default_icecast_bitrate", "string", &opt_default_icecast_bitrate,
+    "48000" },
+  { "path_distconf",  "path",    &opt_path_distconf,  "config.dist" },
   { "path_acl",       "path",    &opt_path_acl,       "acl" },
   { "path_webacl",    "path",    &opt_path_webacl,    "webacl" },
   { "path_logfile",   "path",    &opt_path_logfile,   "log" },
@@ -82,12 +91,14 @@ t_opts opt_opts[] = {
   { "path_playout",   "path",    &opt_path_playout,   "player.out" },
   { "path_idea",      "path",    &opt_path_idea,      "idea" },
   { "path_mixer",     "path",    &opt_path_mixer,     "/dev/mixer" },
-  { "path_language",  "path",    &opt_path_language, DATADIR "/english.lang"},
+  { "path_language",  "path",    &opt_path_language,  DATADIR "/english.lang"},
   { "port",           "int",     &opt_port,           "4444" },
-  { "gap",            "int",     &opt_gap,            "1" },
+  { "gap",            "double",  &opt_gap,            "1" },
   { "play",           "switch",  &opt_play,           "off" },
   { "random",         "switch",  &opt_random,         "off" },
   { "factor",         "double",  &opt_factor,         "0.60" },
+  { "rate_unheard",   "double",  &opt_rate_unheard,   "0.55" },
+  { "rate_unrated",   "double",  &opt_rate_unrated,   "0.50" },
   { "filter",         "string",  &opt_filter,         "" },
   { NULL, NULL, NULL, NULL }
 };
@@ -161,19 +172,5 @@ int opt_read(const char *root)
       return -1;
     }
   }
-  /* special case - player variable is an indirected to another variable */
-  if ((val = conf_getvalue("player")) == NULL) {
-    if (mserv_verbose)
-      printf("No player specified, defaulting to /usr/local/bin/mpg123\n");
-    opt_player = "/usr/local/bin/mpg123";
-  } else {
-    if ((opt_player = conf_getvalue(val)) == NULL) {
-      fprintf(stderr, "%s: player setting '%s' not found\n", progname,
-	      val);
-      return -1;
-    }
-  }
-  if (mserv_verbose)
-    printf("opt special player=%s\n", opt_player);
   return 0;
 }
